@@ -1,3 +1,9 @@
+(******************************************************************************)
+(*     Alt-Ergo: The SMT Solver For Software Verification                     *)
+(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     This file is distributed under the terms of the CeCILL-C licence       *)
+(******************************************************************************)
+
 (******************************************************************************)
 (*     The Alt-Ergo theorem prover                                            *)
 (*     Copyright (C) 2006-2013                                                *)
@@ -1182,7 +1188,7 @@ let axioms_of_rules keep_triggers loc name lf acc env =
       (fun acc (f, _) ->
         let f = Triggers.make keep_triggers false f in
         let name = (Hstring.fresh_string ()) ^ "_" ^ name in
-        let td = {c = TAxiom(loc,name,false,f); annot = new_id () } in
+        let td = {c = TAxiom(loc,name,f); annot = new_id () } in
 	(td, env)::acc
       ) acc lf
   in 
@@ -1195,7 +1201,7 @@ let type_hypothesis keep_triggers acc env_f loc sort f =
   let f = monomorphize_form f in
   let f = Triggers.make keep_triggers false f in
   let td = 
-    {c = TAxiom(loc, fresh_hypothesis_name sort, false,f); 
+    {c = TAxiom(loc, fresh_hypothesis_name sort,f); 
      annot = new_id () } in
   (td, env_f)::acc
     
@@ -1242,11 +1248,11 @@ let type_decl keep_triggers (acc, env) d =
 	let td = {c = TLogic(loc,lp,pp_ty); annot = new_id () } in
 	(td, env)::acc, env'
 
-      | Axiom(loc,name,inv,f) -> 
+      | Axiom(loc,name,f) -> 
 	Options.tool_req 1 "TR-Typing-AxiomDecl$_F$";
 	let f, _ = type_form env f in 
 	let f = Triggers.make keep_triggers false f in
-	let td = {c = TAxiom(loc,name,inv,f); annot = new_id () } in
+	let td = {c = TAxiom(loc,name,f); annot = new_id () } in
 	(td, env)::acc, env
 
       | Rewriting(loc, name, lr) -> 
@@ -1261,7 +1267,7 @@ let type_decl keep_triggers (acc, env) d =
 
       | Goal(loc, n, f) ->
         Options.tool_req 1 "TR-Typing-GoalDecl$_F$";
-	  (*let f = move_up f in*)
+	(*let f = move_up f in*)
 	let f = alpha_renaming_env env f in
  	type_and_intro_goal keep_triggers acc env loc Thm n f, env
 
@@ -1284,9 +1290,9 @@ let type_decl keep_triggers (acc, env) d =
 	let p = {pp_desc=PPapp(n,lvar) ; pp_loc=loc } in
 	let infix = match d with Function_def _ -> PPeq | _ -> PPiff in
 	let f = { pp_desc = PPinfix(p,infix,e) ; pp_loc = loc } in
-	  (* le trigger [[p]] ne permet pas de replier la definition,
-	     donc on calcule les termes maximaux de la definition pour
-	     laisser une possibilite de replier *)
+	(* le trigger [[p]] ne permet pas de replier la definition,
+	   donc on calcule les termes maximaux de la definition pour
+	   laisser une possibilite de replier *)
 	let trs = max_terms e in
 	let f = make_pred loc ([p]::[trs]) f l in
 	let f,_ = type_form env f in
@@ -1348,10 +1354,10 @@ let split_goals l =
 	  | TGoal (_, _, _, _) -> 
 	    ctx, [], [], (x::(local_hyp@global_hyp@ctx))::ret
 	      
-	  | TAxiom (_, s, _, _) when is_global_hyp s ->
+	  | TAxiom (_, s, _) when is_global_hyp s ->
 	    ctx, x::global_hyp, local_hyp, ret
 
-	  | TAxiom (_, s, _, _) when is_local_hyp s ->
+	  | TAxiom (_, s, _) when is_local_hyp s ->
 	    ctx, global_hyp, x::local_hyp, ret
 
 	  | _ -> x::ctx, global_hyp, local_hyp, ret
