@@ -1,3 +1,9 @@
+(******************************************************************************)
+(*     Alt-Ergo: The SMT Solver For Software Verification                     *)
+(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     This file is distributed under the terms of the CeCILL-C licence       *)
+(******************************************************************************)
+
 (******************************************************************************)
 (*     The Alt-Ergo theorem prover                                            *)
 (*     Copyright (C) 2006-2013                                                *)
@@ -14,4 +20,10 @@
 (*   This file is distributed under the terms of the CeCILL-C licence         *)
 (******************************************************************************)
 
-module CX : Sig.X
+module Shostak : Sig.X
+
+module Use : Use.S with type r = Shostak.r
+
+module Uf : Uf.S with type r = Shostak.r
+
+module Relation : Sig.RELATION with type r = Shostak.r and type uf = Uf.t
