@@ -1,3 +1,9 @@
+(******************************************************************************)
+(*     Alt-Ergo: The SMT Solver For Software Verification                     *)
+(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     This file is distributed under the terms of the CeCILL-C licence       *)
+(******************************************************************************)
+
 (******************************************************************************)
 (*     The Alt-Ergo theorem prover                                            *)
 (*     Copyright (C) 2006-2013                                                *)
@@ -22,6 +28,9 @@ module type ALIEN = sig
   val extract : r -> (r abstract) option
 end
 
-module Make 
-  (X : ALIEN) : Sig.THEORY with type r = X.r and type t = X.r abstract
+module Shostak
+  (X : ALIEN) : Sig.SHOSTAK with type r = X.r and type t = X.r abstract
 
+module Relation 
+  (X : ALIEN) (Uf : Uf.S) : Sig.RELATION 
+  with type r = X.r and type uf = Uf.t
